@@ -62,6 +62,10 @@ class InvalidTagException(OctueSDKException, ValueError):
     """Raise when a tag applied to a data file or dataset"""
 
 
+class InvalidLabelException(OctueSDKException, ValueError):
+    """Raise when a label applied to a data file or dataset"""
+
+
 class ServiceNotFound(OctueSDKException):
     """Raise when a Service of the given ID has not been found on the Google Pub/Sub server (i.e. if there is no topic
     associated with the Service ID).
