@@ -25,27 +25,7 @@ class Identifiable:
         """Constructor for Identifiable class"""
         self._name = name
         super().__init__(*args, **kwargs)
-
-        # Store a boolean record of whether this object was created with a previously-existing uuid or was created new.
-        self._created = True if id is None else False
-
-        if isinstance(id, uuid.UUID):
-            # If it's a uuid, stringify it
-            id = str(id)
-
-        elif isinstance(id, str):
-            # If it's a string (or something similar which can be converted to UUID) check it's valid
-            try:
-                id = str(uuid.UUID(id))
-            except ValueError:
-                raise InvalidInputException(f"Value of id '{id}' is not a valid uuid string or instance of class UUID")
-
-        elif id is not None:
-            raise InvalidInputException(
-                f"Value of id '{id}' must be a valid uuid string, an instance of class UUID or None"
-            )
-
-        self._id = id or gen_uuid()
+        self._set_id(id)
 
     def __str__(self):
         return f"{self.__class__.__name__} {self._id}"
@@ -60,3 +40,32 @@ class Identifiable:
     @property
     def name(self):
         return self._name
+
+    def _set_id(self, value):
+        """Set the ID to the given value.
+
+        :param str|uuid.UUID|None value:
+        :return None:
+        """
+        # Store a boolean record of whether this object was created with a previously-existing uuid or was created new.
+        self._created = True if value is None else False
+
+        if isinstance(value, uuid.UUID):
+            # If it's a uuid, stringify it
+            value = str(value)
+
+        elif isinstance(value, str):
+            # If it's a string (or something similar which can be converted to UUID) check it's valid
+            try:
+                value = str(uuid.UUID(value))
+            except ValueError:
+                raise InvalidInputException(
+                    f"Value of id '{value}' is not a valid uuid string or instance of class UUID"
+                )
+
+        elif value is not None:
+            raise InvalidInputException(
+                f"Value of id '{value}' must be a valid uuid string, an instance of class UUID or None"
+            )
+
+        self._id = value or gen_uuid()
