@@ -2,9 +2,10 @@ from unittest.mock import patch
 import google.api_core.exceptions
 
 from octue.cloud.pub_sub.service import Service
-from octue.cloud.pub_sub.subscription import Subscription
+from octue.cloud.pub_sub.subscription import SEVEN_DAYS, THIRTY_ONE_DAYS, Subscription
 from octue.cloud.pub_sub.topic import Topic
 from octue.resources.service_backends import GCPPubSubBackend
+from tests import TEST_PROJECT_NAME
 from tests.base import BaseTestCase
 
 
@@ -40,9 +41,27 @@ class TestSubscription(BaseTestCase):
                 self.subscription.create(allow_existing=False)
 
     def test_create_with_allow_existing_when_already_exists(self):
-        """Test that trying to create a subscription that already exists when `allow_existing` is `True` results in no error."""
+        """Test that trying to create a subscription that already exists when `allow_existing` is `True` results in no
+        error.
+        """
         with patch(
             "octue.cloud.pub_sub.service.pubsub_v1.SubscriberClient.create_subscription",
             side_effect=google.api_core.exceptions.AlreadyExists(""),
         ):
             self.subscription.create(allow_existing=True)
+
+    def test_create(self):
+        """Test that creating a subscription works properly."""
+        service = Service(backend=GCPPubSubBackend(project_name=TEST_PROJECT_NAME))
+        topic = Topic(name="my-topic", namespace="tests", service=service)
+        topic.create(allow_existing=True)
+
+        subscription = Subscription(name="world", topic=topic, namespace="hello", service=service)
+        response = subscription.create(allow_existing=True)
+
+        self.assertEqual(response._pb.ack_deadline_seconds, 60)
+        self.assertEqual(response._pb.expiration_policy.ttl.seconds, THIRTY_ONE_DAYS)
+        self.assertEqual(response._pb.message_retention_duration.seconds, SEVEN_DAYS)
+
+        subscription.delete()
+        topic.delete()
