@@ -7,6 +7,7 @@ from datetime import datetime, timezone
 from unittest.mock import patch
 
 from octue import exceptions
+from octue.cloud import storage
 from octue.cloud.storage import GoogleCloudStorageClient
 from octue.mixins import MixinBase, Pathable
 from octue.resources.datafile import TEMPORARY_LOCAL_FILE_CACHE, Datafile
@@ -60,8 +61,6 @@ class DatafileTestCase(BaseTestCase):
         df = Datafile(path="a_path")
         self.assertTrue(isinstance(df.id, str))
         self.assertEqual(type(uuid.UUID(df.id)), uuid.UUID)
-        self.assertIsNone(df.sequence)
-        self.assertEqual(0, df.cluster)
 
     def test_path_argument_required(self):
         """Ensures instantiation without a path will fail"""
@@ -115,14 +114,12 @@ class DatafileTestCase(BaseTestCase):
         self.assertIn("Extension provided (notcsv) does not match file extension", error.exception.args[0])
 
     def test_file_attributes_accessible(self):
-        """Ensures that its possible to set the sequence, cluster and timestamp"""
+        """Ensures that its possible to set the timestamp"""
         df = self.create_valid_datafile()
         self.assertIsInstance(df.size_bytes, int)
         self.assertGreaterEqual(df._last_modified, 1598200190.5771205)
         self.assertEqual("a_test_file.csv", df.name)
 
-        df.sequence = 2
-        df.cluster = 0
         df.timestamp = 0
 
     def test_cannot_set_calculated_file_attributes(self):
@@ -144,12 +141,10 @@ class DatafileTestCase(BaseTestCase):
         serialised_datafile = self.create_valid_datafile().serialise()
 
         expected_fields = {
-            "cluster",
             "id",
             "name",
             "path",
             "timestamp",
-            "sequence",
             "tags",
             "labels",
             "_cloud_metadata",
@@ -172,7 +167,7 @@ class DatafileTestCase(BaseTestCase):
     def test_is_in_cloud(self):
         """Test whether a file is in the cloud or not can be determined."""
         self.assertFalse(self.create_valid_datafile().is_in_cloud)
-        self.assertTrue(Datafile(path="gs://hello/file.txt").is_in_cloud)
+        self.assertTrue(Datafile(path="gs://hello/file.txt", hypothetical=True).is_in_cloud)
 
     def test_from_cloud_with_bare_file(self):
         """Test that a Datafile can be constructed from a file on Google Cloud storage with no custom metadata."""
@@ -184,13 +179,11 @@ class DatafileTestCase(BaseTestCase):
             path_in_bucket=path_in_bucket,
         )
 
-        datafile = Datafile.from_cloud(
-            project_name=TEST_PROJECT_NAME, bucket_name=TEST_BUCKET_NAME, datafile_path=path_in_bucket
+        datafile = Datafile(
+            path=storage.path.generate_gs_path(TEST_BUCKET_NAME, path_in_bucket), project_name=TEST_PROJECT_NAME
         )
 
         self.assertEqual(datafile.path, f"gs://{TEST_BUCKET_NAME}/{path_in_bucket}")
-        self.assertEqual(datafile.cluster, 0)
-        self.assertEqual(datafile.sequence, None)
         self.assertEqual(datafile.tags, TagDict())
         self.assertEqual(datafile.labels, LabelSet())
         self.assertTrue(isinstance(datafile.size_bytes, int))
@@ -201,76 +194,40 @@ class DatafileTestCase(BaseTestCase):
         """Test that a Datafile can be constructed from a file on Google Cloud storage with custom metadata."""
         datafile, project_name, bucket_name, path_in_bucket, _ = self.create_datafile_in_cloud(
             timestamp=datetime.now(tz=timezone.utc),
-            cluster=0,
-            sequence=1,
             labels={"blah-shah-nah", "blib", "glib"},
             tags={"good": True, "how_good": "very"},
         )
-        gs_path = f"gs://{TEST_BUCKET_NAME}/{path_in_bucket}"
-        downloaded_datafile = Datafile.from_cloud(project_name, cloud_path=gs_path)
 
+        gs_path = f"gs://{TEST_BUCKET_NAME}/{path_in_bucket}"
+
+        downloaded_datafile = Datafile(path=gs_path, project_name=project_name)
         self.assertEqual(downloaded_datafile.path, gs_path)
         self.assertEqual(downloaded_datafile.id, datafile.id)
         self.assertEqual(downloaded_datafile.timestamp, datafile.timestamp)
         self.assertEqual(downloaded_datafile.hash_value, datafile.hash_value)
-        self.assertEqual(downloaded_datafile.cluster, datafile.cluster)
-        self.assertEqual(downloaded_datafile.sequence, datafile.sequence)
         self.assertEqual(downloaded_datafile.tags, datafile.tags)
         self.assertEqual(downloaded_datafile.labels, datafile.labels)
         self.assertEqual(downloaded_datafile.size_bytes, datafile.size_bytes)
         self.assertTrue(isinstance(downloaded_datafile._last_modified, float))
 
-    def test_from_cloud_with_overwrite(self):
-        """Test that a datafile can be instantiated from the cloud and have its attributes overwritten if new values
-        are given in kwargs.
-        """
-        datafile, project_name, bucket_name, path_in_bucket, _ = self.create_datafile_in_cloud()
-
-        new_id = str(uuid.uuid4())
-
-        downloaded_datafile = Datafile.from_cloud(
-            project_name=project_name,
-            bucket_name=bucket_name,
-            datafile_path=path_in_bucket,
-            allow_overwrite=True,
-            id=new_id,
-        )
-
-        self.assertEqual(downloaded_datafile.id, new_id)
-        self.assertNotEqual(datafile.id, downloaded_datafile.id)
-
-    def test_from_cloud_with_overwrite_when_disallowed_results_in_error(self):
-        """Test that attempting to overwrite the attributes of a datafile instantiated from the cloud when not allowed
-        results in an error.
-        """
-        _, project_name, bucket_name, path_in_bucket, _ = self.create_datafile_in_cloud()
-
-        with self.assertRaises(exceptions.AttributeConflict):
-            Datafile.from_cloud(
-                project_name=project_name,
-                bucket_name=bucket_name,
-                datafile_path=path_in_bucket,
-                allow_overwrite=False,
-                id=str(uuid.uuid4()),
-            )
-
     def test_to_cloud_updates_cloud_metadata(self):
         """Test that calling Datafile.to_cloud on a datafile that is already cloud-based updates its metadata in the
         cloud.
         """
-        datafile, project_name, bucket_name, path_in_bucket, _ = self.create_datafile_in_cloud(cluster=0)
+        datafile, project_name, bucket_name, path_in_bucket, _ = self.create_datafile_in_cloud(labels={"start"})
 
-        datafile.cluster = 3
+        datafile.labels = {"finish"}
         datafile.to_cloud(project_name, bucket_name=bucket_name, path_in_bucket=path_in_bucket)
 
         self.assertEqual(
-            Datafile.from_cloud(project_name, bucket_name=bucket_name, datafile_path=path_in_bucket).cluster, 3
+            Datafile(storage.path.generate_gs_path(bucket_name, path_in_bucket), project_name=project_name).labels,
+            {"finish"},
         )
 
     def test_to_cloud_does_not_update_cloud_metadata_if_update_cloud_metadata_is_false(self):
         """Test that calling Datafile.to_cloud with `update_cloud_metadata=False` doesn't update the cloud metadata."""
-        datafile, project_name, bucket_name, path_in_bucket, _ = self.create_datafile_in_cloud(cluster=0)
-        datafile.cluster = 3
+        datafile, project_name, bucket_name, path_in_bucket, _ = self.create_datafile_in_cloud(labels={"start"})
+        datafile.labels = {"finish"}
 
         with patch("octue.resources.datafile.Datafile.update_cloud_metadata") as mock:
             datafile.to_cloud(
@@ -279,14 +236,15 @@ class DatafileTestCase(BaseTestCase):
             self.assertFalse(mock.called)
 
         self.assertEqual(
-            Datafile.from_cloud(project_name, bucket_name=bucket_name, datafile_path=path_in_bucket).cluster, 0
+            Datafile(storage.path.generate_gs_path(bucket_name, path_in_bucket), project_name=project_name).labels,
+            {"start"},
         )
 
     def test_to_cloud_does_not_update_metadata_if_no_metadata_change_has_been_made(self):
         """Test that Datafile.to_cloud does not try to update cloud metadata if no metadata change has been made."""
-        _, project_name, bucket_name, path_in_bucket, _ = self.create_datafile_in_cloud(cluster=0)
+        _, project_name, bucket_name, path_in_bucket, _ = self.create_datafile_in_cloud(labels={"start"})
 
-        datafile = Datafile.from_cloud(project_name, bucket_name=bucket_name, datafile_path=path_in_bucket)
+        datafile = Datafile(storage.path.generate_gs_path(bucket_name, path_in_bucket), project_name=project_name)
 
         with patch("octue.resources.datafile.Datafile.update_cloud_metadata") as mock:
             datafile.to_cloud()
@@ -306,12 +264,12 @@ class DatafileTestCase(BaseTestCase):
         provided.
         """
         _, project_name, bucket_name, path_in_bucket, _ = self.create_datafile_in_cloud()
-        datafile = Datafile.from_cloud(project_name, bucket_name=bucket_name, datafile_path=path_in_bucket)
+        datafile = Datafile(storage.path.generate_gs_path(bucket_name, path_in_bucket), project_name=project_name)
         datafile.to_cloud()
 
     def test_to_cloud_does_not_try_to_update_file_if_no_change_has_been_made_locally(self):
         """Test that Datafile.to_cloud does not try to update cloud file if no change has been made locally."""
-        datafile, project_name, bucket_name, path_in_bucket, _ = self.create_datafile_in_cloud(cluster=0)
+        datafile, project_name, bucket_name, path_in_bucket, _ = self.create_datafile_in_cloud(labels={"start"})
 
         with patch("octue.cloud.storage.client.GoogleCloudStorageClient.upload_file") as mock:
             datafile.to_cloud()
@@ -321,11 +279,12 @@ class DatafileTestCase(BaseTestCase):
         """Test that a cloud datafile's metadata can be updated."""
         _, project_name, bucket_name, path_in_bucket, _ = self.create_datafile_in_cloud()
 
-        new_datafile = Datafile(path="glib.txt", cluster=32)
+        new_datafile = Datafile(path="glib.txt", labels={"new"})
         new_datafile.update_cloud_metadata(project_name, bucket_name=bucket_name, path_in_bucket=path_in_bucket)
 
         self.assertEqual(
-            Datafile.from_cloud(project_name, bucket_name=bucket_name, datafile_path=path_in_bucket).cluster, 32
+            Datafile(storage.path.generate_gs_path(bucket_name, path_in_bucket), project_name=project_name).labels,
+            {"new"},
         )
 
     def test_update_cloud_metadata_works_with_implicit_cloud_location_if_cloud_location_previously_provided(self):
@@ -334,12 +293,13 @@ class DatafileTestCase(BaseTestCase):
         """
         _, project_name, bucket_name, path_in_bucket, _ = self.create_datafile_in_cloud()
 
-        datafile = Datafile.from_cloud(project_name, bucket_name=bucket_name, datafile_path=path_in_bucket)
-        datafile.cluster = 32
+        datafile = Datafile(storage.path.generate_gs_path(bucket_name, path_in_bucket), project_name=project_name)
+        datafile.labels = {"new"}
         datafile.update_cloud_metadata()
 
         self.assertEqual(
-            Datafile.from_cloud(project_name, bucket_name=bucket_name, datafile_path=path_in_bucket).cluster, 32
+            Datafile(storage.path.generate_gs_path(bucket_name, path_in_bucket), project_name=project_name).labels,
+            {"new"},
         )
 
     def test_update_cloud_metadata_raises_error_if_no_cloud_location_provided_and_datafile_not_from_cloud(self):
@@ -354,7 +314,7 @@ class DatafileTestCase(BaseTestCase):
     def test_get_local_path(self):
         """Test that a file in the cloud can be temporarily downloaded and its local path returned."""
         _, project_name, bucket_name, path_in_bucket, contents = self.create_datafile_in_cloud()
-        datafile = Datafile.from_cloud(project_name, bucket_name=bucket_name, datafile_path=path_in_bucket)
+        datafile = Datafile(storage.path.generate_gs_path(bucket_name, path_in_bucket), project_name=project_name)
 
         with open(datafile.get_local_path()) as f:
             self.assertEqual(f.read(), contents)
@@ -362,7 +322,7 @@ class DatafileTestCase(BaseTestCase):
     def test_get_local_path_with_cached_file_avoids_downloading_again(self):
         """Test that attempting to download a cached file avoids downloading it again."""
         _, project_name, bucket_name, path_in_bucket, _ = self.create_datafile_in_cloud()
-        datafile = Datafile.from_cloud(project_name, bucket_name=bucket_name, datafile_path=path_in_bucket)
+        datafile = Datafile(storage.path.generate_gs_path(bucket_name, path_in_bucket), project_name=project_name)
 
         # Download for first time.
         datafile.get_local_path()
@@ -402,7 +362,7 @@ class DatafileTestCase(BaseTestCase):
     def test_open_with_reading_cloud_file(self):
         """Test that a cloud datafile can be opened for reading."""
         _, project_name, bucket_name, path_in_bucket, contents = self.create_datafile_in_cloud()
-        datafile = Datafile.from_cloud(project_name, bucket_name=bucket_name, datafile_path=path_in_bucket)
+        datafile = Datafile(storage.path.generate_gs_path(bucket_name, path_in_bucket), project_name=project_name)
 
         with datafile.open() as f:
             self.assertEqual(f.read(), contents)
@@ -410,7 +370,7 @@ class DatafileTestCase(BaseTestCase):
     def test_open_with_writing_to_cloud_file(self):
         """Test that a cloud datafile can be opened for writing and that both the remote and local copies are updated."""
         _, project_name, bucket_name, path_in_bucket, original_contents = self.create_datafile_in_cloud()
-        datafile = Datafile.from_cloud(project_name, bucket_name=bucket_name, datafile_path=path_in_bucket)
+        datafile = Datafile(storage.path.generate_gs_path(bucket_name, path_in_bucket), project_name=project_name)
 
         new_file_contents = "nanana"
 
@@ -452,8 +412,6 @@ class DatafileTestCase(BaseTestCase):
         self.assertEqual(datafile.absolute_path, deserialised_datafile.absolute_path)
         self.assertEqual(datafile.hash_value, deserialised_datafile.hash_value)
         self.assertEqual(datafile.size_bytes, deserialised_datafile.size_bytes)
-        self.assertEqual(datafile.cluster, deserialised_datafile.cluster)
-        self.assertEqual(datafile.sequence, deserialised_datafile.sequence)
 
     def test_deserialise_uses_path_from_if_path_is_relative(self):
         """Test that Datafile.deserialise uses the path_from parameter if the datafile's path is relative."""
@@ -492,7 +450,7 @@ class DatafileTestCase(BaseTestCase):
         datafile = Datafile(path="hello.txt", timestamp=None)
         self.assertIsNone(datafile.posix_timestamp)
 
-        datafile.timestamp = datetime(1970, 1, 1)
+        datafile.timestamp = datetime(1970, 1, 1, tzinfo=timezone.utc)
         self.assertEqual(datafile.posix_timestamp, 0)
 
     def test_datafile_as_context_manager(self):
@@ -508,22 +466,19 @@ class DatafileTestCase(BaseTestCase):
             self.assertEqual(f.read(), contents)
 
     def test_from_datafile_as_context_manager(self):
-        """Test that Datafile.from_cloud can be used as a context manager to manage cloud changes."""
+        """Test that Datafile can be used as a context manager to manage cloud changes."""
         _, project_name, bucket_name, path_in_bucket, original_content = self.create_datafile_in_cloud()
         new_contents = "Here is the new content."
         self.assertNotEqual(original_content, new_contents)
 
-        with Datafile.from_cloud(project_name, bucket_name=bucket_name, datafile_path=path_in_bucket, mode="w") as (
-            datafile,
-            f,
-        ):
+        path = storage.path.generate_gs_path(bucket_name, path_in_bucket)
+
+        with Datafile(path, project_name=project_name, mode="w") as (datafile, f):
             datafile.add_labels("blue")
             f.write(new_contents)
 
         # Check that the cloud metadata has been updated.
-        re_downloaded_datafile = Datafile.from_cloud(
-            project_name, bucket_name=bucket_name, datafile_path=path_in_bucket
-        )
+        re_downloaded_datafile = Datafile(path, project_name=project_name)
         self.assertTrue("blue" in re_downloaded_datafile.labels)
 
         # The file cache must be cleared so the modified cloud file is downloaded.
@@ -542,8 +497,6 @@ class DatafileTestCase(BaseTestCase):
             {
                 "octue__id",
                 "octue__timestamp",
-                "octue__cluster",
-                "octue__sequence",
                 "octue__tags",
                 "octue__labels",
                 "octue__sdk_version",
@@ -552,5 +505,5 @@ class DatafileTestCase(BaseTestCase):
 
         self.assertEqual(
             datafile.metadata(use_octue_namespace=False).keys(),
-            {"id", "timestamp", "cluster", "sequence", "tags", "labels", "sdk_version"},
+            {"id", "timestamp", "tags", "labels", "sdk_version"},
         )
